import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import com.slackandassociates.Utilities;

/** Class used to wrap the tetris panel up into a frame or applet
 * panel.<br><br>
 * <b>Changes:</b>
 * <ul>
 * <li> 2002-05-31 - Initial release.
 * <li> 2002-06-01 - Added 'centering' code to frame creation.  Made pieces
 *                   create with random rotation.
 * <li> 2002-06-22 - Replaced centering code with a call to 'centerWindowOnScreen'
 *                   method in Utilities class.
 * <li> 2002-12-04 - Modified what 'addKeyListener' was applied to so it would 
 *                   work under java 1.4+ also.
 * <li> 2002-12-07 - Added focuslistener to tetrispanel to restore focus if lost
 *                   (work-around for java 1.4x).
 * <li> 2002-12-09 - Removed 1.4+ fixes (fixed within JTetrisPanel).
 * </ul>
 * @author Michael G. Slack
 * @author slack@attglobal.net
 * @version Version 1.04
*/
public class JTetris extends JApplet
{
    private JFrame frame = null;
    private Container frame_cp = null;
    private JTetrisPanel pnl = new JTetrisPanel();

    /** Method used to create the frame if ran from 'main' method. */ 
    private void createFrame()
    {
        frame = new JFrame("JTetris - 2002");
        // set default close operation
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }

    /** Method implementing applet initialization routine.  It also
     * manages the setup of the panels, contents and frames.
    */
    public void init()
    {
        // get content pane
        if (frame != null) {
            frame_cp = frame.getContentPane();
        }
        else {
            frame_cp = this.getContentPane();
            frame_cp.setLayout(new BorderLayout());
        }

        // add game panel
        frame_cp.add(pnl, BorderLayout.CENTER);

        // other stuff
        if (frame != null) {
            // add keystroke listener
            frame.addKeyListener(pnl);
            // prepare to show
            frame.pack();
            Utilities.centerWindowOnScreen(frame);
            frame.setResizable(false);
            // show me now
            frame.setVisible(true);
        }
        else {
            this.addKeyListener(pnl);
        }
    }

    /** Method used to emulate the process of 'starting' an applet from
     * a 'main' method.
    */
    public void go()
    {
        init();
        start(); // in case we want to add something here...
    }

    /** Method used to run the game as a client application.
    */
    public static void main(String[] args)
    {
        JTetris tet = new JTetris();

        tet.createFrame();
        tet.go();
    }
}