package tetrisbean;

import java.awt.*;

/**
 * The BoardEvent class.
 *
 * IBM DeveloperWorks
 * Scott Clee (Scott_Clee@uk.ibm.com)
 * Software Engineer, IBM
 * March 2002
 * Updated April 11, 2002
 *
 * Updated 2002-04-21 M.G.S.
 *  - Added a board event type (constants and processing).
 *  - Created another constructor to handle setting event type.
 *
 * @author Scott Clee
 */
public class BoardEvent extends Event
{
    public static final int PIECE_ADDED = 0;
    public static final int ROW_REMOVED = 1;
    public static final int UNKNOWN     = 2;

    private Object fSource;
    private int fType = UNKNOWN;

    /**
     * Create a BoardEvent.
     * 
     * @param source The source of the event.
     */
    public BoardEvent(Object source)
    {
        this(source, UNKNOWN);
    }

    /**
     * Create a BoardEvent with a given type.
     *
     * @param source The source of the event.
     * @param type The type of the board event.
     */
    public BoardEvent(Object source, int type)
    {
        super(null, 0, source);
        fSource = source;
        if ((type >= PIECE_ADDED) && (type <= UNKNOWN))
            fType = type;
        else
            fType = UNKNOWN;
    }

    /**
     * Returns the source of the event.
     * 
     * @return The source of the event.
     */
    public Object getSource()
    {
        return fSource;
    }

    /**
     * Returns the type of the board event.
     *
     * @return The type of the event.
     */
    public int getType()
    {
        return fType;
    }
}
